use crate::db::{db_get_posts, db_new_post};
use crate::ServerState;

use actix_web::web::Data;
use actix_web::{get, post, web::Json, HttpResponse, Responder, Result};
use log::info;
use serde::{Deserialize, Serialize};
use sqlx::FromRow;

// The post as it is received from the client
// The token is used to identify the user
#[derive(Debug, Serialize, Deserialize)]
pub struct NewPost {
    pub content: String,
    pub token: String,
}

// The post as it is stored in the database, with all the related metadata
#[derive(Debug, Serialize, Deserialize, Clone, FromRow)]
pub struct Post {
    pub id: i64,
    pub user_id: i64,
    pub content: String,
    pub upvotes: i64,
    pub downvotes: i64,
    pub created_at: chrono::NaiveDateTime,
    pub updated_at: chrono::NaiveDateTime,
}

#[derive(Debug, Serialize, Deserialize, Clone, FromRow)]
pub struct User {
    pub id: i64,
    pub username: String,
    pub password: String,
    pub created_at: chrono::NaiveDateTime,
    pub updated_at: chrono::NaiveDateTime,
}

#[get("/posts")]
pub async fn get_posts(state: Data<ServerState>) -> Result<impl Responder> {
    Ok(HttpResponse::Ok().json(db_get_posts(&state.pool).await))
}

#[post("/posts")]
pub async fn new_post(new_post: Json<NewPost>, state: Data<ServerState>) -> Result<impl Responder> {
    return match db_new_post(new_post.into_inner(), &state.pool).await {
        Some(post) => {
            info!("Created post {:?}", post.id);
            Ok(HttpResponse::Ok().json(post))
        }
        None => Ok(HttpResponse::InternalServerError().json("Error")),
    };
}
